package xyz.leomurca.csgomatches.data.remote

import io.mockk.MockKAnnotations
import io.mockk.coEvery
import io.mockk.coVerify
import io.mockk.every
import io.mockk.impl.annotations.InjectMockKs
import io.mockk.impl.annotations.RelaxedMockK
import io.mockk.mockk
import junit.framework.TestCase.assertTrue
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.ExperimentalCoroutinesApi
import kotlinx.coroutines.test.StandardTestDispatcher
import kotlinx.coroutines.test.resetMain
import kotlinx.coroutines.test.runTest
import kotlinx.coroutines.test.setMain
import kotlinx.serialization.json.Json
import org.junit.After
import org.junit.Before
import org.junit.Test
import retrofit2.Response
import xyz.leomurca.csgomatches.data.model.ErrorDto
import xyz.leomurca.csgomatches.data.model.LeagueDto
import xyz.leomurca.csgomatches.data.model.MatchDto
import xyz.leomurca.csgomatches.data.model.SerieDto
import xyz.leomurca.csgomatches.data.model.TeamDetailsDto
import xyz.leomurca.csgomatches.domain.model.Resource
import xyz.leomurca.csgomatches.utils.ApiDefaults.ITEMS_PER_PAGE
import kotlin.test.assertEquals

@OptIn(ExperimentalCoroutinesApi::class)
class MatchRemoteDataSourceImplTest {

    @RelaxedMockK
    private lateinit var api: MatchesApiService

    @RelaxedMockK
    private lateinit var json: Json

    @InjectMockKs
    private lateinit var dataSource: MatchRemoteDataSourceImpl

    @Before
    fun setUp() {
        MockKAnnotations.init(this)
        Dispatchers.setMain(StandardTestDispatcher())
    }

    @After
    fun tearDown() {
        Dispatchers.resetMain()
    }

    @Test
    fun `upcomingMatches - response is successful - return Resource Success with matches`() =
        runTest {
            // Arrange
            val matches = listOf(MATCH_DTO)
            coEvery { api.upcomingMatches(page = 1) } returns Response.success(matches)

            // Act
            val result = dataSource.upcomingMatches(page = 1)

            // Assert
            coVerify(exactly = 1) {
                api.upcomingMatches("running, not_started", false, "begin_at", ITEMS_PER_PAGE, 1)
            }
            assertTrue(result is Resource.Success)
            assertEquals(matches, (result as Resource.Success).data)
        }

    @Test
    fun `upcomingMatches - response is an error - return api error with error body`() = runTest {
        // Arrange
        val errorDto = ErrorDto(message = "API failure")

        coEvery { api.upcomingMatches(page = 1) } returns mockk(relaxed = true) {
            every { isSuccessful } returns false
        }
        every { json.decodeFromString<ErrorDto>(any()) } returns errorDto

        // Act
        val result = dataSource.upcomingMatches(page = 1)

        // Assert
        assertTrue(result is Resource.Error)
        assertEquals("API failure", (result as Resource.Error).message)
    }


    @Test
    fun `upcomingMatches - response is an RuntimeException - returns Resource Error`() = runTest {
        // Arrange
        coEvery { api.upcomingMatches(page = 1) } throws RuntimeException("Network error")

        // Act
        val result = dataSource.upcomingMatches(page = 1)

        // Assert
        assertTrue(result is Resource.Error)
        assertEquals("Network error", (result as Resource.Error).message)
    }

    @Test
    fun `teamDetails - response is successful - returns Resource Success`() = runTest {
        // Arrange
        val teamDetails = listOf(TEAM_DETAILS_DTO)
        coEvery { api.teamDetails("123") } returns Response.success(teamDetails)

        // Act
        val result = dataSource.teamDetails("123")

        // Assert
        assertTrue(result is Resource.Success)
        assertEquals(teamDetails, (result as Resource.Success).data)
    }


    @Test
    fun `teamDetails - response is an error - returns Resource Error`() = runTest {
        // Arrange
        val errorDto = ErrorDto(message = "Team not found")

        coEvery { api.teamDetails("456") } returns mockk(relaxed = true) {
            every { isSuccessful } returns false
        }
        every { json.decodeFromString<ErrorDto>(any()) } returns errorDto

        // Act
        val result = dataSource.teamDetails("456")

        // Assert
        assertTrue(result is Resource.Error)
        assertEquals("Team not found", (result as Resource.Error).message)
    }

    @Test
    fun `teamDetails - response is RuntimeException - returns Resource Error`() = runTest {
        // Arrange
        coEvery { api.teamDetails("789") } throws RuntimeException("Unknown error")

        // Act
        val result = dataSource.teamDetails("789")

        // Assert
        assertTrue(result is Resource.Error)
        assertEquals("Unknown error", (result as Resource.Error).message)
    }

    companion object {
        private val MATCH_DTO = MatchDto(
            beginAt = null,
            opponents = emptyList(),
            league = LeagueDto(
                id = 5078,
                name = "United21",
                imageUrl = "https://cdn.pandascore.co/images/league/image/5078/800px-united21_allmode-png"
            ),
            serie = SerieDto(
                id = 9519,
                fullName = "Season 35 2025"
            ),
            status = "not_started"
        )

        private val TEAM_DETAILS_DTO = TeamDetailsDto(
            id = 123L,
            name = "Team 1",
            imageUrl = "",
            players = emptyList()
        )
    }
}